/*
3D Mesh Editer via "A Simple Approach to Interactive Free-Form 
Shape Deformations" 
Copyright:(c) Shin Yoshizawa, 2002
E-mail: shin.yoshizawa@mpi-sb.mpg.de
URL: http://www.mpi-sb.mpg.de/~shin
Affiliation: Max-Planck-Institut fuer Informatik: Computer Graphics Group 
 Stuhlsatzenhausweg 85, 66123 Saarbruecken, Germany
 Phone +49 681 9325-414 Fax +49 681 9325-499 

 All right is reserved by Shin Yoshizawa.
This java program and sources are allowed for only primary user of 
research and educational purposes. Don't use secondary: copy, distribution, 
diversion, business purpose, and etc.. 
 */
import java.awt.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class Vertex implements java.io.Serializable{
  
  Point3d p;
  double localmindis;
  double H;
  double absH;
  int ID;
  DoubleVector normal;
  DoubleVector Hn;
  DoubleVector laplace;
  DoubleVector normalizedL;
  int neighborV;
  IDList VHead;
  IDList VTail;
  int boundary;
  int neighborF;
  IDList FHead;
  IDList FTail;
  float color;
  int neighborI;
  IDList IHead;
  IDList ITail;

  public Vertex()
    {
      p = new Point3d(0.0,0.0,0.0);
      normal = new DoubleVector(0.0,0.0,0.0);
      laplace = new DoubleVector(0.0,0.0,0.0);
      normalizedL = new DoubleVector(0.0,0.0,0.0);
      Hn  = new DoubleVector(0.0,0.0,0.0);
      neighborV=0;neighborF=0;neighborI=0;
      this.listinitialize();
    }
  public Vertex(double dx,double dy,double dz)
    {
      p = new Point3d(dx,dy,dz);
      normal = new DoubleVector(0.0,0.0,0.0);
      laplace = new DoubleVector(0.0,0.0,0.0);
      normalizedL = new DoubleVector(0.0,0.0,0.0);
       Hn  = new DoubleVector(0.0,0.0,0.0);
      neighborV=0;neighborF=0;neighborI=0;
      this.listinitialize();
    }
  public Vertex(double dx,double dy,double dz,int dID)
    {
     p = new Point3d(dx,dy,dz);
      ID = dID;
      normal = new DoubleVector(0.0,0.0,0.0);
      laplace = new DoubleVector(0.0,0.0,0.0);
      normalizedL = new DoubleVector(0.0,0.0,0.0);
       Hn  = new DoubleVector(0.0,0.0,0.0);
      neighborV=0;neighborF=0;neighborI=0;
      this.listinitialize();
    }
  public void listinitialize(){
    System.gc();
    neighborV=0;neighborF=0;neighborI=0;
    VHead = new IDList();
    VTail = new IDList();
    this.Init(VHead,VTail);
    FHead = new IDList();
    FTail = new IDList();
    this.Init(FHead,FTail);
    IHead = new IDList();
    ITail = new IDList();
    this.Init(IHead,ITail);
    
  }
  public void LapNormalize(){
    double dum = Math.sqrt(((laplace.x*laplace.x)
			    +(laplace.y*laplace.y)
			    +(laplace.z*laplace.z)));
    if(dum!=0.0){
      normalizedL.x = ((laplace.x)/dum);
      normalizedL.y = ((laplace.y)/dum);
      normalizedL.z = ((laplace.z)/dum);
    }else{
      normalizedL.x = (laplace.x);
      normalizedL.y = (laplace.y);
      normalizedL.z = (laplace.z);

    } 
  }
  private void Init(IDList h,IDList t)
    {
      h.next = new IDList();
      t.back = new IDList();
      h.next = t;
      t.back = h;
    }
   private IDList next(IDList now)
    {
      return (now.next);
    }
  private IDList back(IDList now)
    {
      return (now.back);
    }
  
  public void AppendV(int dID)
    {
      IDList now = new IDList(dID);
      
      IDList dummy = VTail.back;
      now.next = VTail;
      VTail.back = now;
      now.back = dummy;
      dummy.next =now;
      neighborV++;
    }
  public void AppendF(int dID)
    {
      IDList now = new IDList(dID);
      
      IDList dummy = FTail.back;
      now.next = FTail;
      FTail.back = now;
      now.back = dummy;
      dummy.next =now;
      neighborF++;
    }
  private boolean SearchI(int dID){
    
    IDList now = IHead;
    while(next(now)!=ITail){
      now = next(now);
      if((now.ID==dID)){
	return true;
      }
    }
    return false;
  }
  public void AppendI(int dID)
    {
      if(dID!=ID)
	if(!SearchI(dID)){
	  IDList now = new IDList(dID);
	  IDList dummy = ITail.back;
	  now.next = ITail;
	  ITail.back = now;
	  now.back = dummy;
	  dummy.next =now;
	  neighborI++;
	}
    }
  
  
}
